/***************************************************
//Web: http://www.buydisplay.com
EastRising Technology Co.,LTD
****************************************************/
#include <stdlib.h>
#include "ER-EPD026-1B.h"
#include "Debug.h"
#include "GUI_Paint.h"



/******************************************************************************
function :	Software reset
parameter:
******************************************************************************/
void EPD2IN6::EPD_Reset(void)
{
    EPD_RST_1;
    Dev->Dev_Delay_ms(200);
    EPD_RST_0;
    Dev->Dev_Delay_ms(200);
    EPD_RST_1;
    Dev->Dev_Delay_ms(200);
}

/******************************************************************************
function :	send command
parameter:
    Reg : Command register
******************************************************************************/
void EPD2IN6::EPD_SendCommand(UBYTE Reg)
{
    EPD_DC_0;
    EPD_CS_0;
    SPI_Write_Byte(Reg);
    EPD_CS_1;
}

/******************************************************************************
function :	send data
parameter:
    Data : Write data
******************************************************************************/
void EPD2IN6::EPD_SendData(UBYTE Data)
{
    EPD_DC_1;
    EPD_CS_0;
    SPI_Write_Byte(Data);
    EPD_CS_1;
}

/******************************************************************************
function :	Wait until the busy_pin goes LOW
parameter:
******************************************************************************/
void EPD2IN6::EPD_WaitUntilIdle(void)
{
    while(EPD_BUSY_RD == 0) {      //LOW: idle, HIGH: busy
        Dev->Dev_Delay_ms(100);
    }
}
/******************************************************************************
function :	Turn On Display
parameter:
******************************************************************************/
void EPD2IN6::EPD_TurnOnDisplay(void)
{
    EPD_SendCommand(DISPLAY_REFRESH);
    EPD_WaitUntilIdle();
}


/******************************************************************************
function :	Initialize the e-Paper register
parameter:
******************************************************************************/
UBYTE EPD2IN6::EPD_Init(void)
{
    Dev->System_Init();

    EPD_Reset();
    
    EPD_SendCommand(BOOSTER_SOFT_START);
    EPD_SendData(0x17);
    EPD_SendData(0x17);
    EPD_SendData(0x17);

    EPD_SendCommand(POWER_SETTING);
    EPD_SendData(0x03);
    EPD_SendData(0x00);
    EPD_SendData(0x2B);
    EPD_SendData(0x2B);
    EPD_SendData(0x09);
   
    EPD_SendCommand(POWER_ON);
    EPD_WaitUntilIdle();

    EPD_SendCommand(PANEL_SETTING);
    EPD_SendData(0xDF);

    EPD_SendCommand(TCON_RESOLUTION);
    EPD_SendData(0x98);
    EPD_SendData(0x01);
    EPD_SendData(0x28);
 
    EPD_SendCommand(VCOM_AND_DATA_INTERVAL_SETTING);
    EPD_SendData(0xF7);  
    
    return 0;
}

/******************************************************************************
function :	Clear screen
parameter:
******************************************************************************/
void EPD2IN6::EPD_Clear(void) 
{
    UWORD Width, Height;
    Width = (EPD_WIDTH % 8 == 0)? (EPD_WIDTH / 8 ): (EPD_WIDTH / 8 + 1);
    Height = EPD_HEIGHT;
    
    EPD_SendCommand(DATA_START_TRANSMISSION_1);
    Dev->Dev_Delay_ms(2);
    for (UWORD j = 0; j < Height; j++) {
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(0XFF);
        }
    }
    Dev->Dev_Delay_ms(2);
    
    EPD_SendCommand(DATA_START_TRANSMISSION_2);           
    Dev->Dev_Delay_ms(2);
    for (UWORD j = 0; j < Height; j++) {
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(0XFF);
        }
    }
}

/******************************************************************************
function :	Sends the image buffer in RAM to e-Paper and displays
parameter:
******************************************************************************/
void EPD2IN6::EPD_Display(void)
{
    UWORD Width, Height;
    Width = (EPD_WIDTH % 8 == 0)? (EPD_WIDTH / 8 ): (EPD_WIDTH / 8 + 1);
    Height = EPD_HEIGHT;
    
    EPD_SendCommand(DATA_START_TRANSMISSION_1);
    Dev->Dev_Delay_ms(2);
    for (UWORD j = 0; j < Height; j++) {
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(0XFF);
        }
    }
    Dev->Dev_Delay_ms(2);
    
    EPD_SendCommand(DATA_START_TRANSMISSION_2);           
    Dev->Dev_Delay_ms(2);
    for (UWORD j = 0; j < Height; j++) {
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(spiram->SPIRAM_RD_Byte(i + j * Width));
        }
    }
    
    EPD_TurnOnDisplay();
}

/******************************************************************************
function :	Enter sleep mode
parameter:
******************************************************************************/
void EPD2IN6::EPD_Sleep(void) 
{
    EPD_SendCommand(DEEP_SLEEP);
    EPD_SendData(0xa5);
}
